/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.apache.crunch.types.avro;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertNotSame;
import static org.junit.Assert.assertSame;
import static org.junit.Assert.assertTrue;

import com.google.common.collect.ImmutableList;
import org.apache.avro.Schema;
import org.apache.avro.generic.GenericData;
import org.apache.crunch.Pair;
import org.apache.crunch.test.Person;
import org.apache.crunch.test.StringWrapper;
import org.apache.hadoop.conf.Configuration;
import org.junit.Test;

import com.google.common.collect.Lists;

import java.util.List;

public class AvroTableTypeTest {

  @Test
  public void testGetDetachedValue() {
    Integer integerValue = 42;
    Person person = new Person();
    person.name = "John Doe";
    person.age = 42;
    person.siblingnames = Lists.<CharSequence> newArrayList();

    Pair<Integer, Person> pair = Pair.of(integerValue, person);

    AvroTableType<Integer, Person> tableType = Avros.tableOf(Avros.ints(),
        Avros.specifics(Person.class));
    tableType.initialize(new Configuration());

    Pair<Integer, Person> detachedPair = tableType.getDetachedValue(pair);

    assertSame(integerValue, detachedPair.first());
    assertEquals(person, detachedPair.second());
    assertNotSame(person, detachedPair.second());
  }

  @Test
  public void testUnionValueType() {
    List<Schema> schemas = Lists.newArrayList();
    schemas.add(Schema.create(Schema.Type.BOOLEAN));
    schemas.add(Schema.create(Schema.Type.INT));
    Schema union = Schema.createUnion(schemas);
    boolean success = false;
    try {
      Avros.tableOf(Avros.longs(), Avros.generics(union));
      success = true;
    } catch (Exception shouldNotBeThrown) {
    }
    assertTrue("Union type was properly made nullable", success);
  }

  @Test
  public void testIsReflect_ContainsReflectKey() {
    assertTrue(Avros.tableOf(Avros.reflects(StringWrapper.class), Avros.ints()).hasReflect());
  }

  @Test
  public void testIsReflect_ContainsReflectValue() {
    assertTrue(Avros.tableOf(Avros.ints(), Avros.reflects(StringWrapper.class)).hasReflect());
  }

  @Test
  public void testReflect_NoReflectKeyOrValue() {
    assertFalse(Avros.tableOf(Avros.ints(), Avros.ints()).hasReflect());
  }

}
